package info.textgrid.lab.log;

import info.textgrid.lab.conf.OfflineException;
import info.textgrid.lab.conf.OnlineStatus;
import info.textgrid.lab.conf.client.ConfClient;
import info.textgrid.middleware.confclient.ConfservClientConstants;

import java.io.PrintWriter;
import java.io.StringWriter;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

/**
 * Class keeping track of the running logsession 
 */
public final class logsession {
	private int logLevel = 0;
	private String sessionId;
	private static logsession session;

	public static logsession getInstance() {
		if (session == null) {
			session = new logsession();
		}
		return session;
	}

	/**
	 * Returns the loginfo-String, which can be passed to services
	 * using the textgridlogger
	 * 
	 * @return
	 */
	public String getloginfo() {
		if(this.logLevel > 0){
			try {
				return this.getLogEndpoint() + " " + sessionId + " " + logLevel;
			} catch (OfflineException e) {
				OnlineStatus.netAccessFailed("logsession: Could not fetch Log-Service-Endpoint from Confserver!", e);
			}
		}
		return "";
		
	}

	/**
	 * Returns the LOG_SERVICE Endpoint from confclient
	 * @return
	 * @throws OfflineException
	 */
	public String getLogEndpoint() throws OfflineException {
		return ConfClient.getInstance().getValue(ConfservClientConstants.LOG_SERVICE);
	}

	/**
	 * Returns the LOG_SERVICE Endpoint from confclient
	 * @return the log service's endpoint or "" if the endpoint is not available from the configuration server.
	 */
	public String getLogEndpointNoThrow() {
		try {
			return ConfClient.getInstance().getValue(ConfservClientConstants.LOG_SERVICE);
		} catch (OfflineException e) {
			OnlineStatus.netAccessFailed("logsession: Could not fetch Log-Service-Endpoint from Confserver!", e);
		}
		return "";
	}

	/**
	 * Pass the loglevel and logsessionid (which is generated by LOG_SERVICE) for the logsession
	 * this function should only be used from {@link info.textgrid.lab.log.views.logview}
	 * @param logLevel
	 * @param logSessionId
	 */
	public void setLogLevel(int logLevel, String logSessionId) {	
		this.logLevel = logLevel;
		this.sessionId = logSessionId;
		
		StringWriter stringWriter = new StringWriter();
		PrintWriter printWriter = new PrintWriter(stringWriter);
		printWriter.printf("Log view configuration changed: log level = %d, log session id = \"%s\", log service endpoint = \"%s\"",
				logLevel, logSessionId, getLogEndpointNoThrow());
		IStatus status = new Status(IStatus.INFO, Activator.PLUGIN_ID, 
				stringWriter.toString());
		Activator.getDefault().getLog().log(status);
	}
}
